@@ -1,14 +1,14 @@
 #ifndef CATEGORICALDATAEXCHANGE_H_
 #define CATEGORICALDATAEXCHANGE_H_
 /**@file CategoricalDataExchange.h
- * @brief 
+ * @brief
  */
 #include "IDataExchange.h"
 #include "coclust/src/Models/CategoricalLBModel.h"
 class CategoricalDataExchange: public IDataExchange
 {
   public:
-    CategoricalDataExchange(){};
+    CategoricalDataExchange():a_(1), b_(1) {};
     virtual void dataOutput(Rcpp::S4& obj,ICoClustModel*,bool);
     virtual void dataInput(Rcpp::S4 & obj);
     virtual void instantiateModel(ICoClustModel*& model);
@@ -16,7 +16,7 @@ class CategoricalDataExchange: public IDataExchange
     virtual ~CategoricalDataExchange(){};
   protected:
     MatrixInt m_Dataij_;
-    int a_,b_;
+    STK::Real a_,b_;
 };
 
 #endif /* CATEGORICALDATAEXCHANGE_H_ */
