@@ -38,13 +38,20 @@
 class ContinuousLBModelequalsigma: public ICoClustModel
 {
   public:
-    ContinuousLBModelequalsigma( MatrixReal const& m_Dataij,ModelParameters const& Mparam);
-    ContinuousLBModelequalsigma(MatrixReal const& m_Dataij,VectorInt const & rowlabels,
-                       VectorInt const & collabels,ModelParameters const& Mparam);
+    ContinuousLBModelequalsigma( MatrixReal const& m_Dataij
+                               , ModelParameters const& Mparam
+                               );
+    ContinuousLBModelequalsigma( MatrixReal const& m_Dataij
+                               , VectorInt const & rowlabels
+                               , VectorInt const & collabels
+                               , ModelParameters const& Mparam
+                               );
     /** cloning */
     virtual ContinuousLBModelequalsigma* clone(){return new ContinuousLBModelequalsigma(*this);}
-    virtual void logSumRows(MatrixReal & _m_sum);
-    virtual void logSumCols(MatrixReal & _m_sum);
+
+    virtual void logSumRows(MatrixReal & m_sum);
+    virtual void logSumCols(MatrixReal & m_sum);
+
     virtual void mStepFull();
     virtual bool emRows();
     virtual bool cemRows();
@@ -54,16 +61,16 @@ class ContinuousLBModelequalsigma: public ICoClustModel
     virtual bool semCols();
     virtual bool GibbsRows();
     virtual bool GibbsCols();
+
+    virtual STK::Real computeLnLikelihood();
+    virtual bool initStopCriteria();
     virtual void parameterStopCriteria();
-    virtual STK::Real estimateLikelihood();
-    virtual bool cemInitStep();
-    virtual bool emInitStep();
-    virtual void finalizeOutput();
     virtual void consoleOut();
-    virtual void modifyThetaStart();
-    virtual void copyThetaStart();
-    virtual void copyThetaMax();
-    virtual void modifyThetaMax();
+
+    virtual void saveThetaInit();
+    virtual void modifyTheta();
+    virtual void copyTheta();
+
     /** @return the number of free parameters of the distribution of a block.*/
     virtual int nbFreeParameters() const;
 
@@ -74,25 +81,23 @@ class ContinuousLBModelequalsigma: public ICoClustModel
     MatrixReal const& mean() const;
     /**Return Sigma ContinuousLBModelequalsigma::m_Sigma2kl_*/
     STK::Real sigma2() const;
+
   protected:
     MatrixReal const& m_Dataij_;
     MatrixReal m_ClusterDataij_;
     MatrixReal m_Dataij2_;
-    MatrixReal m_Mukl_; //,m_Mukl2_;
-    STK::Real Sigma2_,Sigma2start_,Sigma2max_;
-    MatrixReal m_Muklold1_,m_Muklold2_,m_Muklstart_,m_Muklmax_;
-    MatrixReal m_Vjk1_,m_Vjk2_;
-    MatrixReal m_Uil1_,m_Uil2_;
+    MatrixReal m_Mukl_;
+    STK::Real Sigma2_,Sigma2temp_;
+    MatrixReal m_Muklold1_,m_Muklold2_,m_Mukltemp_;
+    MatrixReal m_Vjk2_, m_Uil2_;
 
     //M-steps
-    void mStepRows();
-    void mStepCols();
-
-    // Functions used to operate on data in intermediate steps when running the Initialization
-    bool initCEMCols();
-    bool initEMCols();
-    void selectRandomRowsFromData(MatrixReal &);
-    void generateRandomMean(const MatrixReal & , MatrixReal &);
+    virtual void mStepRows();
+    virtual void mStepCols();
+    /** Compute m_Vjk_ array for all models */
+    virtual void computeVjk();
+    /** Compute m_Uil_ array for all models */
+    virtual void computeUil();
 };
 
 inline MatrixReal const& ContinuousLBModelequalsigma::mean() const
@@ -103,19 +108,29 @@ inline STK::Real ContinuousLBModelequalsigma::sigma2() const
 
 inline void ContinuousLBModelequalsigma::mStepRows()
 {
-  if(!Mparam_.fixedproportions_) { v_logPiek_=(v_Tk_/nbSample_).log();}
-
-  m_Mukl_  = (m_Tik_.transpose()*m_Uil1_)/(v_Tk_*v_Rl_.transpose());
-  //m_Mukl2_ = m_Mukl_.square();
+  mSteplogPiek();
+  m_Mukl_  = (m_Tik_.transpose()*m_Uil_)/(v_Tk_*v_Rl_.transpose());
   Sigma2_  = ((m_Tik_.transpose()*m_Uil2_).sum()-v_Tk_.dot(m_Mukl_.square()*v_Rl_))/dimprod_;
 }
 
 inline void ContinuousLBModelequalsigma::mStepCols()
 {
-  if(!Mparam_.fixedproportions_) { v_logRhol_=(v_Rl_/nbVar_).log();}
-
-  m_Mukl_  = (m_Vjk1_.transpose()*m_Rjl_)/(v_Tk_*v_Rl_.transpose());
-//  m_Mukl2_ = m_Mukl_.square();
+  mSteplogRhol();
+  m_Mukl_  = (m_Vjk_.transpose()*m_Rjl_)/(v_Tk_*v_Rl_.transpose());
   Sigma2_  = ((m_Vjk2_.transpose()*m_Rjl_).sum()-v_Tk_.dot(m_Mukl_.square()*v_Rl_))/dimprod_;
 }
+
+inline void ContinuousLBModelequalsigma::computeUil()
+{
+  m_Uil_ = m_Dataij_*m_Rjl_;
+  m_Uil2_ = m_Dataij2_*m_Rjl_;
+}
+
+inline void ContinuousLBModelequalsigma::computeVjk()
+{
+  m_Vjk_ = m_Dataij_.transpose()*m_Tik_;
+  m_Vjk2_ = m_Dataij2_.transpose()*m_Tik_;
+}
+
+
 #endif /* CONTINUOUSLBMODELEQUALSIGMA_H_ */
